import 'dart:io';

import 'package:async/async.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ChobiAdMob {

  static const androidTestAdaptiveBannerId = 'ca-app-pub-3940256099942544/9214589741';
  static const iosTestAdaptiveBannerId = 'ca-app-pub-3940256099942544/2435281174';

  static const androidTestInterstitialId = 'ca-app-pub-3940256099942544/1033173712';
  static const iosTestInterstitialId = 'ca-app-pub-3940256099942544/4411468910';

  static const androidTestMovieInterstitialId = 'ca-app-pub-3940256099942544/8691691433';
  static const iosTestMovieInterstitialId = 'ca-app-pub-3940256099942544/5135589807';

  static String getIdString({required String id, required String testId, bool isTest = true}) {
    return isTest ? testId : id;
  }

  static String? getIdStringByPlatform({required String androidId, required String testAndroidId, required String iosId, required String testIosId, bool isTest = true}) {
    if (Platform.isAndroid) {
      return getIdString(id: androidId, testId: testAndroidId, isTest: isTest);
    } else if (Platform.isIOS) {
      return getIdString(id: iosId, testId: testIosId, isTest: isTest);
    } else {
      return null;
    }
  }


  static String? getAdaptiveBannerId({required String androidId, required String iosId,  bool isTest = true}) {
    return getIdStringByPlatform(androidId: androidId, testAndroidId: androidTestAdaptiveBannerId, iosId: iosId, testIosId: iosTestAdaptiveBannerId, isTest: isTest);
  }

  static String? getInterstitialId({required String androidId, required String iosId,  bool isTest = true}) {
    return getIdStringByPlatform(androidId: androidId, testAndroidId: androidTestInterstitialId, iosId: iosId, testIosId: iosTestInterstitialId, isTest: isTest);
  }

  static String? getMovieInterstitialId({required String androidId, required String iosId,  bool isTest = true}) {
    return getIdStringByPlatform(androidId: androidId, testAndroidId: androidTestMovieInterstitialId, iosId: iosId, testIosId: iosTestMovieInterstitialId, isTest: isTest);
  }


  static Future<int> loadLastClickedTimeFromPrefs({required String prefKey}) async {
    final prefs = await SharedPreferences.getInstance();
    return prefs.getInt(prefKey) ?? 0;
  }

  static ConsentRequestParameters createConsentRequestParameters({required bool isDebug, List<String>? testIds}) {
    final settings = ConsentDebugSettings(
      debugGeography: isDebug ? DebugGeography.debugGeographyEea : null,
      testIdentifiers: testIds,
    );
    return ConsentRequestParameters()
        ..consentDebugSettings = settings;
  }


  static void checkConsentFormAvailable({
    required ConsentRequestParameters params,
    Function(bool isFormAvailable)? onCheckFinished,
    Function(FormError)? onError,
  }) {
    ConsentInformation.instance.requestConsentInfoUpdate(
      params,
          () async {
        final isFormAvailable = await ConsentInformation.instance.isConsentFormAvailable();
        onCheckFinished?.call(isFormAvailable);
          },
          (error) => onError?.call(error),
    );
  }

  static void loadConsentForm({Function(ConsentStatus status)? onCheckedStatus, Function(FormError)? onError}) {
    ConsentForm.loadConsentForm(
          (consentForm) async {
            final status = await ConsentInformation.instance.getConsentStatus();
            if (status == ConsentStatus.required) {
              consentForm.show((formError) {
                if (formError != null) {
                  onError?.call(formError);
                }

                loadConsentForm(onCheckedStatus: onCheckedStatus, onError: onError);
              });
            } else {
              onCheckedStatus?.call(status);
            }
          },
          (formError) => onError?.call(formError),
    );
  }

  static void simpleCheckAndLoadForm({
    required ConsentRequestParameters params,
    Function(ConsentStatus status)? onCheckedStatus,
    Function(FormError)? onError,
  }) {
    checkConsentFormAvailable(
      params: params,
      onCheckFinished: (isFormAvailable) {
        if (isFormAvailable) {
          loadConsentForm(
            onCheckedStatus: onCheckedStatus,
            onError: onError,
          );
        } else {
          onCheckedStatus?.call(ConsentStatus.obtained);
        }
      },
      onError: onError,
    );
  }

  static Future<void> resetConsentStatus() async {
    await ConsentInformation.instance.reset();
  }



  //---
  late int _adLastClickedTime;
  final String lastClickedPrefsKey;
  final bool isDebug;
  final List<String>? testDeviceIds;
  final int adIntervalTimeMs;

  final String? androidId;
  final String? iosId;
  final String androidTestId;
  final String iosTestId;

  int get intervalWaitMs => adIntervalTimeMs - (DateTime.now().millisecondsSinceEpoch - _adLastClickedTime);

  void Function(ConsentStatus status) onConsentCheckFinished = (_) {};
  void Function(FormError error) onConsentCheckError = (_) {};

  CancelableOperation<bool>? _waitingOperation;


  String? getAdId() {
    return getIdStringByPlatform(
      androidId: androidId ?? androidTestId,
      testAndroidId: androidTestId,
      iosId: iosId ?? iosTestId,
      testIosId: iosTestId,
    );
  }


  ChobiAdMob({
    int adLastClickedTime = 0,
    required this.lastClickedPrefsKey,
    this.isDebug = true,
    this.testDeviceIds,
    this.androidId,
    this.iosId,
    required this.androidTestId,
    required this.iosTestId,
    this.adIntervalTimeMs = 5 * 60 * 1000,
  }) {
    _adLastClickedTime = adLastClickedTime;
  }

  void _innerCheckProcess() {
    final params = createConsentRequestParameters(
      isDebug: isDebug,
      testIds: testDeviceIds,
    );

    simpleCheckAndLoadForm(
      params: params,
      onCheckedStatus: onConsentCheckFinished,
      onError: onConsentCheckError,
    );
  }

  Future<int> updateLastClickedTime() async {
    final time = DateTime.now().millisecondsSinceEpoch;
    _adLastClickedTime = time;
    final prefs = await SharedPreferences.getInstance();
    await prefs.setInt(lastClickedPrefsKey, time);
    return time;
  }


  Future<void> stopAdProcess() async {
    await _waitingOperation?.cancel();
  }

  Future<void> checkPermissionAndStartAdProcess() async {

    await _waitingOperation?.cancel();

    final waitingMs = intervalWaitMs;

    if (waitingMs <= 0) {
      _innerCheckProcess();
    } else {
      _waitingOperation = CancelableOperation.fromFuture(
        Future.delayed(Duration(milliseconds: waitingMs), () => true),
        onCancel: () async {
          _waitingOperation = null;
          return false;
        },
      );

      _waitingOperation?.value.then((value) {
        if (value) {
          _waitingOperation = null;
          checkPermissionAndStartAdProcess();
        }
      });
    }
  }
}
